@@ -22,6 +22,7 @@
 #define GLOM_DATASTRUCTURE_FIELD_H
 
 #include <libgdamm.h>
+#include "../sharedptr.h"
 
 //Predicate, for use with std::find_if():
 
@@ -43,6 +44,11 @@ public:
     return (element.get_name() == m_strName);
   }
 
+  bool operator() (const sharedptr<T_Element>& element)
+  {
+    return (element->get_name() == m_strName);
+  }
+
 protected:
   Glib::ustring m_strName;
 };
@@ -141,6 +147,11 @@ public:
    */
   Glib::ustring sql(const Gnome::Gda::Value& value) const;
 
+  /** Get a suitable operator to use when finding records.
+   * For instance, == for numbers, or LIKE for text.
+   */
+  Glib::ustring sql_find_operator() const;
+
   Glib::ustring get_calculation() const;
   void set_calculation(const Glib::ustring& calculation);
   bool get_has_calculation() const;
