@@ -66,8 +66,7 @@ void ComboGlomChoicesBase::set_choices_with_second(const type_list_values_with_s
 
       if(m_with_second)
       {
-        row[m_Columns.m_col_second] = GlomConversions::get_text_for_gda_value(m_layoutitem_second.m_field.get_glom_type(), iter->second, layout_item->m_numeric_format);;
-         g_warning("ComboGlomChoicesBase::set_choices_with_second(): second=%s", iter->second.to_string().c_str());
+        row[m_Columns.m_col_second] = GlomConversions::get_text_for_gda_value(m_layoutitem_second.m_field.get_glom_type(), iter->second, layout_item->m_numeric_format);
       }
     }
   }
