@@ -36,18 +36,26 @@ EntryGlom::EntryGlom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::X
   m_glom_type(Field::TYPE_TEXT)
 {
   setup_menu();
+  init();
 }
 
 EntryGlom::EntryGlom(Field::glom_field_type glom_type)
 : m_glom_type(glom_type)
 {
   setup_menu();
+  init();
 }
 
 EntryGlom::~EntryGlom()
 {
 }
 
+void EntryGlom::init()
+{
+  if(m_glom_type == Field::TYPE_NUMERIC)
+    set_alignment(1.0); //Align numbers to the right.
+}
+
 void EntryGlom::set_glom_type(Field::glom_field_type glom_type)
 {
   m_glom_type = glom_type;
@@ -84,11 +92,6 @@ void EntryGlom::check_for_change()
   }
 }
 
-EntryGlom::type_signal_edited EntryGlom::signal_edited()
-{
-  return m_signal_edited;
-}
-
 bool EntryGlom::on_focus_out_event(GdkEventFocus* event)
 {
   bool result = Gtk::Entry::on_focus_out_event(event);
